Enum quadrature_decoder::Change
source · #[repr(i8)]pub enum Change {
Positive = 1,
Negative = -1,
}Expand description
The change detected by a quadrature decoder.
Variants
Positive = 1
Channel A leads channel B, commonly describing a forwards change.
Negative = -1
Channel B leads channel A, commonly describing a backwards change.